@@ -18,11 +18,12 @@
 #
 # Refer to the README and COPYING files for full details of the license
 #
+import __builtin__
 import os
 from datetime import datetime
 from functools import partial
+import io
 import time
-from xml.dom import minidom
 
 import ethtool
 
@@ -30,7 +31,8 @@ from vdsm import ipwrapper
 from vdsm import netconfpersistence
 from vdsm import netinfo
 from vdsm.netinfo import (getBootProtocol, getDhclientIfaces, BONDING_MASTERS,
-                          BONDING_OPT, _randomIfaceName, _getBondingOptions)
+                          BONDING_OPT, _getBondingOptions, OPERSTATE_UP)
+from vdsm.tool.dump_bonding_defaults import _random_iface_name
 
 from functional import dummy, veth
 from ipwrapperTests import _fakeTypeDetection
@@ -93,6 +95,23 @@ class TestNetinfo(TestCaseBase):
         for s, addr in zip(inputs, ip):
             self.assertEqual(addr, netinfo.ipv6StrToAddress(s))
 
+    def testValidNicSpeed(self):
+        values = ((0,           OPERSTATE_UP, 0),
+                  (-10,         OPERSTATE_UP, 0),
+                  (2 ** 16 - 1, OPERSTATE_UP, 0),
+                  (2 ** 32 - 1, OPERSTATE_UP, 0),
+                  (123,         OPERSTATE_UP, 123),
+                  ('',          OPERSTATE_UP, 0),
+                  ('',          'unknown',    0),
+                  (123,         'unknown',    0))
+
+        for passed, operstate, expected in values:
+            with MonkeyPatchScope([(__builtin__, 'open',
+                                    lambda x: io.BytesIO(str(passed))),
+                                   (netinfo, 'operstate',
+                                    lambda x: operstate)]):
+                self.assertEqual(netinfo.nicSpeed('fake_nic'), expected)
+
     @MonkeyPatch(ipwrapper.Link, '_detectType',
                  partial(_fakeTypeDetection, ipwrapper.Link))
     @MonkeyPatch(netinfo, 'networks', lambda: {'fake': {'bridged': True}})
@@ -215,23 +234,6 @@ class TestNetinfo(TestCaseBase):
                              'devices %s is shown in nics %s' % (hiddens,
                                                                  nics))
 
-    def testGetBandwidthQos(self):
-        notEmptyDoc = minidom.parseString("""<bandwidth>
-                            <inbound average='4500' burst='5400' />
-                            <outbound average='4500' burst='5400' peak='101' />
-                          </bandwidth>""")
-        expectedQosNotEmpty = netinfo._Qos(inbound={'average': 4500,
-                                                    'burst': 5400},
-                                           outbound={'average': 4500,
-                                                     'burst': 5400,
-                                                     'peak': 101})
-        emptyDoc = minidom.parseString("<whatever></whatever>")
-
-        self.assertEqual(expectedQosNotEmpty,
-                         netinfo._parseBandwidthQos(notEmptyDoc))
-        self.assertEqual(netinfo._Qos({}, {}),
-                         netinfo._parseBandwidthQos(emptyDoc))
-
     def testGetBootProtocolIfcfg(self):
         deviceName = "___This_could_never_be_a_device_name___"
         ifcfg = ('DEVICE=%s' % deviceName + '\n' + 'ONBOOT=yes' + '\n' +
@@ -351,10 +353,13 @@ class TestNetinfo(TestCaseBase):
         self.assertNotIn('expired', dhcp4)
         self.assertNotIn('expired2', dhcp4)
 
+    @MonkeyPatch(netinfo, 'BONDING_DEFAULTS', netinfo.BONDING_DEFAULTS
+                 if os.path.exists(netinfo.BONDING_DEFAULTS)
+                 else '../vdsm/bonding-defaults.json')
     @ValidateRunningAsRoot
     def testGetBondingOptions(self):
         INTERVAL = '12345'
-        bondName = _randomIfaceName()
+        bondName = _random_iface_name()
 
         with open(BONDING_MASTERS, 'w') as bonds:
             bonds.write('+' + bondName)
