@@ -479,7 +479,6 @@ def _validateNetworkSetup(networks, bondings):
                 raise ConfigNetworkError(ne.ERR_BAD_PARAMS, 'Cannot specify '
                                          'any attribute when removing')
 
-    currentBondings = netinfo.bondings()
     currentNicsSet = set(netinfo.nics())
     for bonding, bondingAttrs in bondings.iteritems():
         Bond.validateName(bonding)
@@ -487,9 +486,6 @@ def _validateNetworkSetup(networks, bondings):
             Bond.validateOptions(bonding, bondingAttrs['options'])
 
         if bondingAttrs.get('remove', False):
-            if bonding not in currentBondings:
-                raise ConfigNetworkError(ne.ERR_BAD_BONDING, "Cannot remove "
-                                         "bonding %s: Doesn't exist" % bonding)
             continue
 
         nics = bondingAttrs.get('nics', None)
@@ -501,7 +497,7 @@ def _validateNetworkSetup(networks, bondings):
                                      "Unknown nics in: %r" % list(nics))
 
 
-def _handleBondings(bondings, configurator):
+def _handleBondings(bondings, configurator, in_rollback):
     """ Add/Edit/Remove bond interface """
     logger = logging.getLogger("_handleBondings")
 
@@ -511,6 +507,16 @@ def _handleBondings(bondings, configurator):
     addition = []
     for name, attrs in bondings.items():
         if 'remove' in attrs:
+            if name not in _netinfo.bondings:
+                if in_rollback:
+                    logger.error(
+                        'Cannot remove bonding %s during rollback: '
+                        'does not exist', name)
+                    continue
+                else:
+                    raise ConfigNetworkError(
+                        ne.ERR_BAD_BONDING,
+                        "Cannot remove bonding %s: does not exist" % name)
             bond = Bond.objectivize(name, configurator, attrs.get('options'),
                                     attrs.get('nics'), mtu=None,
                                     _netinfo=_netinfo,
@@ -526,12 +532,18 @@ def _handleBondings(bondings, configurator):
         bond = Bond.objectivize(name, configurator, attrs.get('options'),
                                 attrs.get('nics'), mtu=None, _netinfo=_netinfo,
                                 destroyOnMasterRemoval='remove' in attrs)
+        if len(bond.slaves) == 0:
+            raise ConfigNetworkError(ne.ERR_BAD_PARAMS, 'Missing required nics'
+                                     ' for bonding device.')
         logger.debug("Editing bond %r with options %s", bond, bond.options)
         configurator.editBonding(bond, _netinfo)
     for name, attrs in addition:
         bond = Bond.objectivize(name, configurator, attrs.get('options'),
                                 attrs.get('nics'), mtu=None, _netinfo=_netinfo,
                                 destroyOnMasterRemoval='remove' in attrs)
+        if len(bond.slaves) == 0:
+            raise ConfigNetworkError(ne.ERR_BAD_PARAMS, 'Missing required nics'
+                                     ' for bonding device.')
         logger.debug("Creating bond %r with options %s", bond, bond.options)
         configurator.configureBond(bond)
 
@@ -633,7 +645,8 @@ def setupNetworks(networks, bondings, **options):
     options = results['request']['options']
 
     logger.debug("Applying...")
-    with ConfiguratorClass(options.get('_inRollback', False)) as configurator:
+    in_rollback = options.get('_inRollback', False)
+    with ConfiguratorClass(in_rollback) as configurator:
         # Remove edited networks and networks with 'remove' attribute
         for network, networkAttrs in networks.items():
             if network in _netinfo.networks:
@@ -663,7 +676,7 @@ def setupNetworks(networks, bondings, **options):
             else:
                 networksAdded.add(network)
 
-        _handleBondings(bondings, configurator)
+        _handleBondings(bondings, configurator, in_rollback)
 
         # We need to use the newest host info
         _netinfo.updateDevices()
